@@ -27,7 +27,7 @@ from collections import namedtuple
 from concurrent.futures import Future
 from functools import lru_cache, partial
 from itertools import zip_longest
-from typing import Any, Callable, List, Optional, Type, Union
+from typing import Any, Callable, List, Optional, Type, TypeVar, Union
 
 import attrs
 from cattrs.errors import ClassValidationError
@@ -55,7 +55,7 @@ from lsprotocol.types import (
 )
 from lsprotocol.types import (
     ApplyWorkspaceEditParams,
-    ConfigurationParams, Diagnostic,
+    Diagnostic,
     DidChangeTextDocumentParams, DidChangeWorkspaceFoldersParams,
     DidCloseTextDocumentParams, DidOpenTextDocumentParams,
     ExecuteCommandParams, InitializeParams, InitializeResult,
@@ -63,13 +63,16 @@ from lsprotocol.types import (
     RegistrationParams, ResponseErrorMessage, SetTraceParams,
     ShowDocumentParams, ShowMessageParams,
     TraceValues, UnregistrationParams, WorkspaceApplyEditResponse,
-    WorkspaceEdit, InitializeResultServerInfoType
+    WorkspaceEdit, InitializeResultServerInfoType,
+    WorkspaceConfigurationParams
 )
 from pygls.uris import from_fs_path
 from pygls.workspace import Workspace
 
 logger = logging.getLogger(__name__)
 
+F = TypeVar("F", bound=Callable)
+
 
 def call_user_feature(base_func, method_name):
     """Wraps generic LSP features and calls user registered feature
@@ -619,9 +622,9 @@ class JsonRPCProtocol(asyncio.Protocol):
         return self.fm.thread()
 
 
-def lsp_method(method_name: str):
-    def decorator(f):
-        f.method_name = method_name
+def lsp_method(method_name: str) -> Callable[[F], F]:
+    def decorator(f: F) -> F:
+        f.method_name = method_name  # type: ignore[attr-defined]
         return f
     return decorator
 
@@ -686,7 +689,9 @@ class LanguageServerProtocol(JsonRPCProtocol, metaclass=LSPMeta):
     def get_result_type(self, method: str) -> Optional[Type]:
         return METHOD_TO_TYPES.get(method, (None, None))[1]
 
-    def apply_edit(self, edit: WorkspaceEdit, label: str = None) -> WorkspaceApplyEditResponse:
+    def apply_edit(
+        self, edit: WorkspaceEdit, label: Optional[str] = None
+    ) -> WorkspaceApplyEditResponse:
         """Sends apply edit request to the client."""
         return self.send_request(WORKSPACE_APPLY_EDIT,
                                  ApplyWorkspaceEditParams(edit=edit, label=label))
@@ -795,12 +800,12 @@ class LanguageServerProtocol(JsonRPCProtocol, metaclass=LSPMeta):
         cmd_handler = self.fm.commands[params.command]
         self._execute_request(msg_id, cmd_handler, params.arguments)
 
-    def get_configuration(self, params: ConfigurationParams,
+    def get_configuration(self, params: WorkspaceConfigurationParams,
                           callback: Optional[ConfigCallbackType] = None) -> Future:
         """Sends configuration request to the client.
 
         Args:
-            params(ConfigurationParams): ConfigurationParams from lsp specs
+            params(WorkspaceConfigurationParams): WorkspaceConfigurationParams from lsp specs
             callback(callable): Callabe which will be called after
                                 response from the client is received
         Returns:
@@ -809,11 +814,11 @@ class LanguageServerProtocol(JsonRPCProtocol, metaclass=LSPMeta):
         """
         return self.send_request(WORKSPACE_CONFIGURATION, params, callback)
 
-    def get_configuration_async(self, params: ConfigurationParams) -> asyncio.Future:
+    def get_configuration_async(self, params: WorkspaceConfigurationParams) -> asyncio.Future:
         """Calls `get_configuration` method but designed to use with coroutines
 
         Args:
-            params(ConfigurationParams): ConfigurationParams from lsp specs
+            params(WorkspaceConfigurationParams): WorkspaceConfigurationParams from lsp specs
         Returns:
             asyncio.Future that can be awaited
         """
@@ -830,10 +835,71 @@ class LanguageServerProtocol(JsonRPCProtocol, metaclass=LSPMeta):
 
         self.notify(LOG_TRACE, params)
 
-    def publish_diagnostics(self, doc_uri: str, diagnostics: List[Diagnostic]) -> None:
-        """Sends diagnostic notification to the client."""
-        self.notify(TEXT_DOCUMENT_PUBLISH_DIAGNOSTICS,
-                    PublishDiagnosticsParams(uri=doc_uri, diagnostics=diagnostics))
+    def _publish_diagnostics_deprecator(
+        self,
+        params_or_uri: Union[str, PublishDiagnosticsParams],
+        diagnostics: Optional[List[Diagnostic]],
+        version: Optional[int],
+        **kwargs
+    ) -> PublishDiagnosticsParams:
+        if isinstance(params_or_uri, str):
+            message = "DEPRECATION: "
+            "`publish_diagnostics("
+            "self, doc_uri: str, diagnostics: List[Diagnostic], version: Optional[int] = None)`"
+            "will be replaced with `publish_diagnostics(self, params: PublishDiagnosticsParams)`"
+            logging.warning(message)
+
+            params = self._construct_publish_diagnostic_type(
+                params_or_uri,
+                diagnostics,
+                version,
+                **kwargs
+            )
+        else:
+            params = params_or_uri
+        return params
+
+    def _construct_publish_diagnostic_type(
+        self,
+        uri: str,
+        diagnostics: Optional[List[Diagnostic]],
+        version: Optional[int],
+        **kwargs
+    ) -> PublishDiagnosticsParams:
+        if diagnostics is None:
+            diagnostics = []
+
+        args = {
+            **{
+                "uri": uri,
+                "diagnostics": diagnostics,
+                "version": version
+            },
+            **kwargs
+        }
+
+        params = PublishDiagnosticsParams(**args)  # type:ignore
+        return params
+
+    def publish_diagnostics(
+        self,
+        params_or_uri: Union[str, PublishDiagnosticsParams],
+        diagnostics: Optional[List[Diagnostic]] = None,
+        version: Optional[int] = None,
+        **kwargs
+    ):
+        """
+        Sends diagnostic notification to the client.
+        Deprecation:
+          `uri`, `diagnostics` and `version` fields will be deprecated
+        """
+        params = self._publish_diagnostics_deprecator(
+            params_or_uri,
+            diagnostics,
+            version,
+            **kwargs
+        )
+        self.notify(TEXT_DOCUMENT_PUBLISH_DIAGNOSTICS, params)
 
     def register_capability(self, params: RegistrationParams,
                             callback: Optional[Callable[[], None]] = None) -> Future:
